<?php

/**
 * @file
 * Hooks provided by the Remote Entity API.
 */

/**
 * @addtogroup hooks
 * @{
 */

/**
 * Provide an entity type via the remote entity API.
 *
 * This is a placeholder for describing further keys for hook_entity_info(),
 * which are introduced by the remote entity API for providing a new entity type
 * in sync with a remote entity.
 *
 * The following core or Entity API keys are used by the remote entity API:
 *
 * - 'controller class': This should be 'RemoteEntityAPIDefaultController' or
 *    a subclass.
 * - 'metadata controller class': This should be
 *   'RemoteEntityAPIDefaultMetadataController' or a subclass.
 *
 * The following core or Entity API keys are extended by the remote entity API:
 *
 * - 'admin ui': If the entity uses RemoteEntityEntityUIController as its
 *   controller, then this may have the following additional keys:
 *    - 'form buttons': An array specifying which buttons should be shown on the
 *      form, in the format 'button_name' => TRUE. Possible button names are:
 *      - 'fetch': Show a button to fetch all remote entities. (Only use this
 *        when you know the number of remote entities is relatively small!)
 *
 * The remote entity API supports the following keys:
 *
 * - 'remote base table': The remote base table or entity type. This is used
 *   in remote queries.
 * - 'remote query defaults callback': (Optional) A function with signature
 *   callback_remote_entity_query_defaults(), which is called on all new
 *   RemoteEntityQuery objects. This may be used to set default conditions or
 *   relationships on queries.
 * - 'remote entity keys': An array of properties on the Drupal entity that map
 *   to properties on the remote data.
 *    - 'remote id': The remote entity property which gives its remote ID.
 *    - 'label': The remote entity property which gives its label, if
 *      remote_entity_entity_label() is being used as the label callback.
 * - 'property map': An array of mappings from local property names to remote
 *   entity properties. Local names will be made available via the Entity
 *   metadata API. For instance, the key-value pair 'foo' => 'bar' will mean
 *   that $wrapper->foo->value() will return the value of $remote_entity->bar.
 * - 'expiry': (optional) Allows local copies of entities to expire, after
 *   which time they are refreshed. If this is not set, local entities never
 *   expire. This should be an array with the following properties:
 *    - 'expiry time': A value in seconds that determines how long a local copy
 *      of a remote entity is considered fresh. After this time, loading the
 *      local copy will result in a remote retrieval of the entity.
 *    - 'purge': A boolean indicating whether or not to delete expired local
 *      entities on cron. Warning: purging local entities will mean that when
 *      they are next retrieved, their local entity ID will change, thus
 *      breaking any entityreference field values and fields on the entities
 *      themselves.
 * - 'remote entity unpack callback': The name of an implementation of
 *    callback_remote_entity_unpack(). This allows the entity's module to set
 *    remote properties on the entity to be set prior to saving.
 * - (TODO) 'remote entity pack callback': The name of an implementation of
 *    callback_remote_entity_pack(). This allows the entity's module to extract
 *    remote properties from the remote entity data when it is loaded from
 *    the remote source. For example, the controller by default extracts the
 *    remote ID and places it in the entity's 'remote_id' property.
 *
 * @see hook_entity_info()
 * @see entity_crud_hook_entity_info()
 */
function remote_entity_hook_entity_info() {

}

/**
 * Act when remote entities are processed to local copies.
 *
 * This is invoked after remote entities have been packed into local copies
 * and saved.
 *
 * @param $entities
 *  An array of entities, keyed by entity ID.
 * @param $entity_type
 *  The type of the entities.
 *
 * @see RemoteEntityAPIDefaultController::process_remote_entities()
 */
function hook_remote_entity_process($entities, $entity_type) {

}

/**
 * Declare information about remote tables for RemoteEntityQuery.
 *
 * Loosely inspired by hook_views_data().
 *
 * WARNING: This system is currently under active development, and may be
 * further changed in the future.
 *
 * @return
 *  An array of data about remote tables, used for building remote queries.
 *  This should be keyed by connection name, where each value is then an array
 *  keyed by remote table name. Each table value is an array of data for that
 *  table containing:
 *  - 'fields': An array whose keys are remote field names. Each value is an
 *    array which itself contains data about the field. The contents of this
 *    will depend on the connection type.
 *  - 'relationships': The contents of this depend on the connection type.
 *
 * @see remote_entity_get_query_table_info()
 */
function hook_remote_entity_query_table_info() {
  $data = array(
    // The connection name.
    'my_connection' => array(
      // The remote table name.
      'mytable' => array(
        // Data about fields. Not all RemoteEntityQuery implementations will
        // necessarily need this, or need this for all contexts. Eg, these may
        // only be needed when used in conditions.
        'fields' => array(
          // The default type is 'string'; only define ones that differ.
          'typecode' => array(
            'type' => 'int',
          ),
          'statecode' => array(
            'type' => 'int',
          ),
        ),
        // Joins from this table to others.
        'relationships' => array(
          // Relationship name. Arbitrary, used in adding to the query.
          'myrelationship' => array(
            // The LHS of the join, starting from this table.
            'base' => array(
              // The field on this table.
              'field' => 'address1_addressid',
            ),
            // The RHS of the join.
            'join' => array(
              'table' => 'customeraddress',
              'field' => 'customeraddressid',
            ),
          ),
        ),
      ),
    ),
  );

  return $data;
}

/**
 * @addtogroup callbacks
 * @{
 */

/**
 * Allows defaults to be set on all RemoteEntityQueries for an entity type.
 *
 * @param RemoteEntityQuery $query
 *  A RemoteEntityQuery, with the base already set by the entity controller.
 *  This is passed by reference and should be altered directly.
 *  The entity info for the base entity can be found in $query->entity_info.
 *
 * @see remote_entity_hook_entity_info()
 * @see RemoteEntityAPIDefaultController::getRemoteEntityQuery()
 */
function callback_remote_entity_query_defaults($query) {
  // We only work with remote entities of property foo equal to bar.
  $query->fieldRemoteCondition('foo', 'bar');
}

/**
 * Allows setting of remote properties on the entity prior to remote saving.
 *
 * For example, the entity may use FieldAPI to collect data from users, which
 * should then be set into the remote entity data and saved remotely.
 *
 * @param $entity_type
 *  The entity type.
 * @param $entity
 *  The entity that is about to be saved remotely.
 *
 * @return
 *  A boolean indicating whether the entity should be saved locally. This will
 *  happen regardless in the case of a remote insert (as the remote ID needs to
 *  be set in the local copy). For a remote update, this is not automatic, and
 *  this value allows this callback to request a save if it has changed
 *  properties on the entity.
 */
function callback_remote_entity_unpack($entity_type, $entity) {

}
